<?php
//
// +----------------------------------------------------------------------+
// | PHP Version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.02 of the PHP license,      |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Author: George Schlossnagle <george@omniti.com>                      |
// +----------------------------------------------------------------------+
//
// $Id: Telnet2.php,v 1.1 2004/03/17 15:56:05 george Exp $

/**
 * Net_Telnet class.  This allows basic expect-style access to plain
 * text network protocols, though it is specifically designed for
 * use with the telnet protocol.
 *
 * @category Net
 * @package Net_Telnet
 * @author George Schlossnagle <george@omniti.com>
 *
 */

class Net_Telnet {
    /**
     * @var string The remote server address.
     * @access private
     */
    private $_address;

    /**
     * @var integer The remote server port.
     * @access private
     */
    private $_port;

    /**
     * @var resource The remote server socket.
     * @access private
     */
    private $_sock;

    /**
     * @var
     * @access static
     */
    static public $CRLF = "\r\n";
    
    /**
     * @var
     * @access public
     */
    public $prompt;

    /**
     * Constructs a Net_Telnet object.
     *
     * @param string The remote server address, with option port
     * @param optional string The prompt to expect from the server.
     * @param optional integer The timeout for the connection attempt.
     * @access public
     */
    public function __construct($address, $prompt=false, $timeout = 5) {
        if(preg_match("/([\w.-]+)(?::(\d+))?/", $address, $matches))
        {
            $this->_address = $matches[1];
            $this->_port = $matches[2]?$matches[2]:23;
        }
        $this->prompt = $prompt;
        $this->_timeout = $timeout;
        $this->connect();
    }

    /**
     * connect is called automatically from the constructor, so
     * we protect it.
     *
     * @access protected
     */
    protected function connect()
    {
        $this->sock = fsockopen($this->_address, $this->_port); 
        stream_set_timeout($this->_sock, $this->_timeout);
    }

    /**
     * Read a socket until we hit EOF or match a string.
     * 
     * @param string The pattern to match (static currently)
     * @access public
     */
    public function read_response_until($str)
    {
        $response = '';
        while(1) {
            $char = fgetc($this->_sock);
            if($char === false) {
                return $response;
            }
            $response .= $char;
            if(strstr($response, $str)) {
                return substr($response, 0, strpos($response, $str));
            }
        }
    }

    /**
     * Write to the socket, remembering to append the local CRLF.
     * 
     * @param string The command to write to the socket.
     * @access public
     */
    public function write_cmd($cmd)
    {
        $cmd .= self::$CRLF;
        fwrite($this->_sock, $cmd);
    }

    /**
     * A convenience method.    Writes a command to the socket, and
     * reads until the next line prompt is returned from the server.
     * Returns the resulting buffer.
     * 
     * @param string The command to write to the socket.
     * @access public
     */
    public function command($buffer) 
    {
        $this->write_cmd($buffer);
        $response = $this->read_response_until("$this->prompt>");
        return $response;
    }
}

/* vim: set sts=2 ts=2 expandtab ai bs=2 : */
?>
